package com.ekexiu.project.bridge.system.service;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.method.InvalidSession;
import org.jfw.apt.web.annotation.method.ResultToNull;
import org.jfw.apt.web.annotation.method.SetSession;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.SessionVal;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.state.StateCode;

import com.ekexiu.project.bridge.mobile.MobilePhoneService;
import com.ekexiu.project.bridge.system.dao.UserDao;
import com.ekexiu.project.bridge.system.po.User;
import com.ekexiu.project.bridge.system.vo.SessionUser;

/**
 * Created by TT on 2018/8/7.
 */
@Path("/sys")
public class SysService {

	public static final String DEFAULT_PW_STR = StringUtil.md5("123456");

	private String regMobilePhoneReplaceKey;
	private String regMobilePhoneContentTemplate;
	private long timeLimitWithRegMobilePhone = 3 * 60 * 1000;

	@Autowrie
	private UserDao userDao;
	@Autowrie
	private MobilePhoneService mobilePhoneService;

	public UserDao getUserDao() {
		return userDao;
	}

	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}

	public MobilePhoneService getMobilePhoneService() {
		return mobilePhoneService;
	}

	public void setMobilePhoneService(MobilePhoneService mobilePhoneService) {
		this.mobilePhoneService = mobilePhoneService;
	}

	@SetSession("JFW_SESSION_LOGIN_USER=result")
	@Path("/login")
	@Post
	public SessionUser login(@JdbcConn Connection con, String account, String pw, String vc, @Nullable @SessionVal(value ="PIC_LOGIN",remove= true) String code,
			@Nullable @SessionVal(value ="TIMEOUT_PIC_LOGIN",remove=true) Long timeout) throws SQLException {
		if (code == null || timeout == null) {
			return null;
		}
		if (!vc.equals(code)) {
			return null;
		}
		if (System.currentTimeMillis() > timeout) {
			return null;
		}
		User user = userDao.login(con, account, StringUtil.md5(pw));
		if (user != null) {
			return makeSessionUser(user);
		}
		return null;
	}

	@Get
	@Path("")
	public SessionUser get(@LoginUser SessionUser user) {
		return user;
	}

	@Get
	@Path("/logout")
	@InvalidSession
	public void logout() {
	}

	@Post
	@Path("/insert")
	public void insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, User user) throws SQLException {
		user.setId(StringUtil.buildUUID());
		user.setActive(true);
		user.setPasswd(DEFAULT_PW_STR);
		user.setType(0);
		user.setCreator(sessionUser.getId());
		user.setModifier(sessionUser.getId());
		this.userDao.insert(con, user);
	}

	@Get
	@Path("/qo")
	public SessionUser query(@JdbcConn Connection con, String id) throws SQLException {
		return makeSessionUser(this.userDao.query(con, id));
	}

	@Post
	@Path("/update")
	public void update(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, User user) throws SQLException {
		user.setModifier(sessionUser.getId());
		this.userDao.update(con, user);
	}

	@Get
	@Path("/ban")
	public void ban(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
		this.userDao.ban(con, sessionUser.getId(), id);
	}

	@Get
	@Path("/pq")
	public PageQueryResult<SessionUser> pageQuery(@JdbcConn Connection con, @Nullable String account, @Nullable String name, @Nullable String comp,
			int pageSize, int pageNo) throws SQLException {
		PageQueryResult<User> pageQueryResult = this.userDao.pageQuery(con, account, name == null ? null : "%" + name + "%",
				comp == null ? null : "%" + comp + "%", pageSize, pageNo);
		List<User> users = pageQueryResult.getData();
		List<SessionUser> sessionUsers = new ArrayList<>();
		if (!users.isEmpty()) {
			for (User user : users) {
				sessionUsers.add(makeSessionUser(user));
			}
			PageQueryResult<SessionUser> ret = new PageQueryResult<>();
			ret.setPageNo(pageQueryResult.getPageNo());
			ret.setPageSize(pageQueryResult.getPageSize());
			ret.setTotal(pageQueryResult.getTotal());
			ret.setData(sessionUsers);
			return ret;
		}
		return null;
	}

	@Get
	@Path("/resetPw")
	public void resetPw(@JdbcConn(true) Connection con, String id) throws SQLException {
		this.userDao.changePw(con, DEFAULT_PW_STR, id);
	}

	@Post
	@Path("/changePw")
	public void changePw(@JdbcConn(true) Connection con, String id, String oldPw, String newPw) throws SQLException, JfwBaseException {
		User user = this.userDao.query(con, id);
		if (user != null && StringUtil.md5(oldPw).equals(StringUtil.md5(user.getPasswd()))) {
			this.userDao.changePw(con, StringUtil.md5(newPw), id);
		} else {
			throw new JfwBaseException(100001, "用户不存在或密码不匹配");
		}
	}

	@Post
	@Path("/notice")
	public void updateNotice(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String cnt) throws SQLException {
		this.userDao.updateNotice(con, cnt, sessionUser.getId());
	}

	@SetSession({ "RESET_PW_MOBILE=result[0]", "RESET_PW_VC=result[1]", "RESET_PW_TO=result[2]" })
	@ResultToNull
	@Get
	@Path("/regmobilephone")
	public Object[] regMobilePhone(@JdbcConn(false) Connection con, String account, @DefaultValue("false") boolean checkExists) throws Exception {
		if (checkExists) {
			User user = this.userDao.queryByAccount(con, account);
			if (null != user) {
				throw new JfwBaseException(-60000, "not found account");
			}
		}
		Random rd = new Random();
		int vi = rd.nextInt(10000);
		String vcode = String.format("%04d", vi);

		this.mobilePhoneService.sendMessage(account, this.regMobilePhoneContentTemplate, this.regMobilePhoneReplaceKey, vcode);
		Object[] result = new Object[3];
		result[0] = account;
		result[1] = vcode;
		result[2] = System.currentTimeMillis() + 5000 + (5 * 60 * 1000);
		return result;
	}

	@Post
	@Path("/resetPasswordWithMobilePhone")
	public boolean resetPassword(@JdbcConn(true) Connection con, String pw, String vc,
			@Nullable @SessionVal(value = "RESET_PW_MOBILE", remove = true) String account,
			@Nullable @SessionVal(value = "RESET_PW_VC", remove = true) String vcode, @Nullable @SessionVal(value = "RESET_PW_TO", remove = true) Long timeout)
			throws SQLException, JfwBaseException {
		if (vcode == null || account == null || timeout == null) {
			return false;
		}
		if (System.currentTimeMillis() > timeout) {
			return false;
		}
		this.userDao.updatePasswdWithMobile(con, StringUtil.md5(pw), account);
		return true;
	}

	private static SessionUser makeSessionUser(User user) {
		SessionUser sessionUser = new SessionUser();
		sessionUser.setAccount(user.getAccount());
		sessionUser.setComp(user.getComp());
		sessionUser.setEmail(user.getEmail());
		sessionUser.setId(user.getId());
		sessionUser.setJob(user.getJob());
		sessionUser.setName(user.getName());
		sessionUser.setPhone(user.getPhone());
		sessionUser.setRemark(user.getRemark());
		sessionUser.setType(user.getType());
		sessionUser.setActive(user.isActive());
		return sessionUser;
	}
}
